<?php

/**
 * @file
 * Contains Drupal\facets\Plugin\facets\url_processor\CoreViewsUrlProcessor.
 */

namespace Drupal\core_views_facets\Plugin\facets\url_processor;

use Drupal\Core\Url;
use Drupal\facets\FacetInterface;
use Drupal\facets\UrlProcessor\UrlProcessorPluginBase;
use Symfony\Component\HttpFoundation\Request;
use Drupal\core_views_facets\Plugin\facets\facet_source\CoreViewsExposedFilter;
use Drupal\core_views_facets\Plugin\facets\facet_source\CoreViewsContextualFilter;
use Drupal\views\Views;


/**
 * A url processor for views exposed filters.
 *
 * The core_views_facets url processor builds urls as the views exposed filters
 * expect them. We have to adhere to the urls that views uses to be able to use
 * the processing trough views.
 *
 * @FacetsUrlProcessor(
 *   id = "core_views_url_processor",
 *   label = @Translation("Core views url processor"),
 *   description = @Translation("Formats the query URL so views can process it."),
 * )
 */
class CoreViewsUrlProcessor extends UrlProcessorPluginBase {

  /**
   * A string that separates the filters in the query string.
   */
  const SEPARATOR = ':';

  /**
   * An array of active filters.
   *
   * @var string[]
   *   An array containing the active filters.
   */
  protected $activeFilters = [];

  /**
   * {@inheritdoc}
   */
  public function buildUrls(FacetInterface $facet, array $results) {
    // No results are found for this facet, so don't try to create urls.
    if (empty($results)) {
      return [];
    }

    $source = $facet->getFacetSource();

    switch (TRUE) {
      case $source instanceof CoreViewsExposedFilter:
        // First get the current list of get parameters.
        $get_params = $this->request->query;

        $views_filter = $source->getViewsFilterDefinition($facet->getFieldIdentifier());
        $views_filter_parameter = empty($views_filter['expose']) ? $facet->getFieldIdentifier() : $views_filter['expose']['identifier'];

        /** @var \Drupal\facets\Result\ResultInterface $result */
        foreach ($results as &$result) {
          $result_get_params = clone $get_params;

          $active_values = $result_get_params->get($views_filter_parameter, $views_filter['expose']['multiple'] ? [] : '', TRUE);

          // If the value is active, remove the filter string from the parameters.
          if ($result->isActive()) {
            if ($views_filter['expose']['multiple']) {
              if (($key = array_search($result->getRawValue(), $active_values)) !== FALSE) {
                unset($active_values[$key]);
              }
            }
            else {
              if ($active_values == $result->getRawValue()) {
                $active_values = '';
              }
            }
          }
          // If the value is not active, add the filter string.
          else {
            if ($views_filter['expose']['multiple']) {
              $active_values = [$result->getRawValue()];
            }
            else {
              $active_values = $result->getRawValue();
            }
          }

          if ($active_values) {
            $result_get_params->set($views_filter_parameter, $active_values);
          }
          else {
            $result_get_params->remove($views_filter_parameter);
          }

          if ($facet->getFacetSource()->isRenderedInCurrentRequest()) {
            $request = \Drupal::request();
          }
          else {
            $request = Request::create('/' . $source->getPath());
          }
          $url = Url::createFromRequest($request);
          $url->setOption('query', $result_get_params->all());

          $result->setUrl($url);
        }

        return $results;
        break;

      case $source instanceof CoreViewsContextualFilter:
        $map = $source->getViewsArgumentsMap();

        $view = Views::getView($source->pluginDefinition['view_id']);
        $view->setDisplay($source->pluginDefinition['view_display']);

        /** @var \Drupal\facets\Result\ResultInterface $result */
        foreach ($results as &$result) {
          $request_arguments = [];
          foreach ($map as $views_argument_id => $current_argument) {
            if ($views_argument_id == $facet->getFieldIdentifier()) {
              $request_arguments[] = $result->isActive() ? $current_argument['neutral_value'] : $result->getRawValue();
            }
            else {
              $request_arguments[] = $current_argument['value'];
            }
          }

          $url = $view->getUrl($request_arguments);
          $url->setOption('query', $this->request->query->all());
          $result->setUrl($url);
        }

        return $results;
        break;

      default:
        return [];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function setActiveItems(FacetInterface $facet) {
    $source = $facet->getFacetSource();

    switch (TRUE) {
      case $source instanceof CoreViewsExposedFilter:

        $views_filter = $source->getViewsFilterDefinition($facet->getFieldIdentifier());
        $views_filter_parameter = empty($views_filter['expose']) ? $facet->getFieldIdentifier() : $views_filter['expose']['identifier'];

        $url_parameters = $this->request->query;

        if ($views_filter['expose']['multiple']) {
          foreach ($url_parameters->get($views_filter_parameter, [], TRUE) as $value) {
            $facet->setActiveItem(trim($value, '"'));
          }
        }
        else {
          $value = $url_parameters->get($views_filter_parameter, NULL, TRUE);
          if (!empty($value)) {
            $facet->setActiveItem($value);
          }
        }

        break;

      case $source instanceof CoreViewsContextualFilter:
        $map = $source->getViewsArgumentsMap();

        if (isset($map[$facet->getFieldIdentifier()])) {
          $current_argument = $map[$facet->getFieldIdentifier()];
          if ($current_argument['value'] != $current_argument['neutral_value']) {
            $facet->setActiveItem($current_argument['value']);
          }
        }
        break;

      default:
        return;
    }
  }

}
